#include <avr/io.h>

char b = 0;

void setupOsc(void)
{
    //16MHz external crystal
    OSC_XOSCCTRL = OSC_FRQRANGE_12TO16_gc | OSC_XOSCSEL_XTAL_16KCLK_gc;
	
    //Enable external oscillator
    OSC_CTRL |= OSC_XOSCEN_bm;
	
    //Wait for clock stabilization
    while(!(OSC_STATUS & OSC_XOSCRDY_bm));
	
    // Selects clock system as external clock
    // through change protection mechanism
    CCP = CCP_IOREG_gc;
    CLK_CTRL = CLK_SCLKSEL_XOSC_gc;
}

void setUpLED(void)
{
	// PD4 = LED output
	PORTD_OUTSET = PIN4_bm;
	PORTD_DIRSET = PIN4_bm;
}

void LEDon(void)
{
	PORTD.OUTCLR = PIN4_bm;	
}

void LEDoff(void)
{
	PORTD.OUTSET = PIN4_bm;
}

void setUpSerial(void)
{
	// PD7 = RS232 TX output
	PORTD_OUTSET = PIN7_bm;
	PORTD_DIRSET = PIN7_bm;
	
	// PD6 = RS232 RX input
	PORTD_OUTCLR = PIN6_bm;
	PORTD_DIRCLR = PIN6_bm; 
	
	// calculate 9600 baud
	// BSEL = (16000000 / (2^0 * 16 * 9600) - 1 = 103,1666 -> BSCALE = 0
	// FBAUD = (16000000 / (2^0 * 16 * (103 + 1)) = 9615.384 -> error acceptable	
	USARTD1_BAUDCTRLB = 0; //Just to be sure that BSCALE is 0
	USARTD1_BAUDCTRLA = 0x67; // 103
	
	// Disable interrupts, just for safety
	USARTD1_CTRLA = 0;
	
	// 8 data bits, no parity and 1 stop bit
	USARTD1_CTRLC = USART_CHSIZE_8BIT_gc;
	
	// enable receive and transmit
	USARTD1_CTRLB = USART_TXEN_bm | USART_RXEN_bm;
}

void sendChar(char c)
{
	// wait til DATA buffer is empty
	while( !(USARTD1_STATUS & USART_DREIF_bm) );
	
	USARTD1_DATA = c;
}

void sendString(const char *text)
{
	while(*text)
		sendChar(*text++);
}

char usart_receiveByte(void)
{
	while( !(USARTD1_STATUS & USART_RXCIF_bm) );
	
	return USARTD1_DATA;
}

int main(void)
{
	char c;
	
	setupOsc();
	setUpLED();
	setUpSerial();
	
    while(1)
    {
		sendString("Enter a character\r\n");
	
		c = usart_receiveByte();
		sendString("Character was:");
		sendChar(c);
		sendString("\r\n\r\n");
		
		if(b) LEDon(); else LEDoff();		
		b = !b;
    }
}

